--[[
************************************************************************

achievement.lua

File date: 2010-01-20T15:51:38Z
File revision: 4
Project revision: 4
Project version: r4


Author: Angelike

************************************************************************
]]--
local AddonName = ...
local AchievementLineIndex = 1;
local FrameCache = UIFrameCache:New("FRAME", "aatLine", UIParent, "aatWatchFrameLineTemplate");
local AchievementLines = {}
local events = {'ACHIEVEMENT_EARNED','TRACKED_ACHIEVEMENT_UPDATE',}--'PLAYER_ENTERING_WORLD',

aat = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0")--"AceConsole-3.0", 
local funcnames = {}
for k, v in pairs(aat) do
	if type(v) == 'function' then
		tinsert(funcnames,k)
	end
end

local L = LibStub("AceLocale-3.0"):GetLocale(AddonName, true)

-- set up local variables
local NormalFontSize, HeaderFontSize, TitleFontSize, FontString
local NormalFontSpacing, HeaderFontSpacing, BorderSpacing
local IconSizeHeader, IconSizeObjective
local AchievementWatchFrame, AchievementWatchAnchor

function aat:debugprint(text)
--[===[@debug@
	print(AddonName .. ": " .. tostring(text))
--@end-debug@]===]
end

function aat:OnInitialize()
	aat:InitDB();
	aat:InitOptions();
	
	hooksecurefunc("RemoveTrackedAchievement", function()
		if not aat:IsEnabled() then 
			return
		end
		aat:AchievementTracker_DisplayAchievements()
	end)
	hooksecurefunc("AddTrackedAchievement", function()
		if not aat:IsEnabled() then 
			return
		end
		aat:AchievementTracker_DisplayAchievements()
	end)
	
	aat:SetEnabledState(aat.db.profile.AddonStatus)
	do
		aat.funcnames = {}
		local found
		for k, v in pairs(aat) do
			if type(v) == 'function' then
				found = nil
				for _,vold in pairs(funcnames) do
					if k == vold then
						found = true
					end
				end
				if not found then
					tinsert(aat.funcnames,k)
				end
			end
		end
		table.sort(aat.funcnames)
	end
	aat:AchievementTracker_CalcFonts()
end

function aat:OnEnable()
	-- Scripts
	-- try to restore old achievements
	aat:AchievementTracker_DisplayAchievements()
	--
	-- Events
	for k in ipairs(events) do
		aat:RegisterEvent(events[k], "Event_" .. events[k])
	end

	WatchFrame_RemoveObjectiveHandler(WatchFrame_HandleDisplayTrackedAchievements)
	
	if aat.db.char.AchievementWatchMinimized then
		aat:AchievementTracker_Minimize()
	else
		aat:AchievementTracker_Maximize()
	end
	WatchFrame_ClearDisplay();
	
	do -- fix hiding POI and Item Buttons
		for i=1, 25 do
			local itembutton = _G["WatchFrameItem" .. i]
			if itembutton then
				itembutton:Hide()
			else
				break;
			end
		end
		
		-- clear poi buttons
		local button
		for i=1, 4 do
			for j=1,25 do
				button = _G["poiWatchFrameLines" .. i .. "_" ..j]
				if button then
					button:Hide()
				end
			end
		end
	end
	
	WatchFrame_Update();
end

function aat:OnDisable()
	-- events
	for k in ipairs(events) do
		aat:UnregisterEvent(events[k])
	end

	aatAchievementWatchFrame:Hide()

	WatchFrame_AddObjectiveHandler(WatchFrame_HandleDisplayTrackedAchievements);
	
	WatchFrame_Update();

end

function aat:InitDB()
	local defaults = {
		profile = {
			AddonStatus = true,
			debug = false,


		-- Tooltip
			Tooltip = {
				Colors = {
					QuestTracker_TooltipTitle = {
						r = 0.75,
						g = 0.61,
						b = 0,
					},
					QuestTracker_TooltipDesc = {
						r = 1.0,
						g = 0.8,
						b = 0,
					},
					GameTooltip_MobTitle = {
						r = 1,
						g = 1,
						b = 1,
					},
					GameTooltip_ItemTitle = {
						r = 1,
						g = 1,
						b = 1,
					},
					GameTooltip_ObjectiveIncomplete = {
						r = 1,
						g = 0.09,
						b = 0,
					},
					GameTooltip_ObjectiveComplete = {
						r = 0.05,
						g = 0.93,
						b = 0.15,
					},
					GameTooltip_ObjectiveStep = {
						r = 0.96,
						g = 1,
						b = 0,
					},
					GameTooltip_NonObjective = {
						r = 1,
						g = 0.64,
						b = 0.03,
					},
				},
				-- Gametooltip
				mob = true,
				item = true,
				GameTooltip_TitelIsQuestcolor = true,
				GameTooltip_ColorObjectives = true,
				GameTooltip_FadeObjectiveColors = true,
				GameTooltip_ObjectiveUseStep = true,
				
				-- dont know
				TooltipTitleDiff = true,
				TooltipObjFade = true,
				TooltipItemFade = true,
				
			},
		-- Achievementtracker
		--	AchievementTracker = {
				Colors = {
					BgColor 	= {
						r	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
						g	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
						b	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
					},
					BgCornerColor 	= {
						r	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
						g	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
						b	= TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
					},
				},
				-- mouse events
				showtooltip = true,
				showtooltiphint = true,
				clickleft = true,
				clickright = true,
				ManageLines = true,
				NumLines = 20,
				Locked	= false,
				ShowHeader	= true,
				alpha		= 1,
				FontSize = 10,
				RemoveFinished 	= true,
				showbg	= true,
				ownbgcolor	= false,
			--},
		},
		char = {
			FramePosition = {
				posx = -264,
				posy = -200,
				point = "TOPRIGHT",
			},
			AchievementWatchMinimized = false,
			AchievementWatchIDs = {},
		},
	}
	aat.db = LibStub("AceDB-3.0"):New("aatDB", defaults, true)

end

function aat:InitOptions()
	local MainOptionsTable	=	{
		type	=	'group',
		name	=	AddonName,
		args	=	{
			general	=	{
				type	=	'group',
				name	=	AddonName,
				args	=	{
					DisableAddon	=	{
						type	=	'toggle',
						order	=	1,
						width	=	'full',
						name	=	L["Enable Addon"],
						desc	=	L["Enable this Addon"],
						get	=	function(info) return aat:IsEnabled()  end,
						set	=	function(info, newval) 
									if aat:IsEnabled() then
										aat.db.profile.AddonStatus = false;
										aat:Disable();
									else
										aat.db.profile.AddonStatus = true;
										aat:Enable();
									end
								end,
					},
					Spacher1	=	{
						type	=	'header',
						order	=	2,
						name	=	'',
						hidden	= function() return not aat:IsEnabled() end,
					},
					LockedTracker	=	{
						type	=	'toggle',
						order	=	3,
						name	=	L["Lock Tracker"],
						desc	=	L["Makes the achievement tracker unmovable"],
						get	=	function(info) return aat.db.profile.Locked end,
						set	=	function(info, newval) aat.db.profile.Locked = newval; AchievementWatchAnchor.UpdateFrame = nil end,
						hidden	= function() return not aat:IsEnabled() end,
					},
					ResetTracker	=	{
						type	=	'execute',
						order	=	4,
						name	=	L["Reset tracker position"],
						desc	=	L["Reset tracker position"],
						func	=	function(info)
										AchievementWatchAnchor:ClearAllPoints()
										AchievementWatchAnchor:SetPoint("TOPRIGHT", "MinimapCluster", "BOTTOMLEFT", -93, -13)
										aat:SaveFramePosition(AchievementWatchAnchor)
									end,
						hidden	= function() return not aat:IsEnabled() end,
						},
		--[[						Spacing1	=	{
							type	=	'header',
							order	=	3,
							name	=	'',
					}, --]]
					ShowATrackerHeader	=	{
						type	=	'toggle',
						order	=	5,
						width	=	'full',
						name	=	L["Show Achievement Tracker Header"],
						desc	=	L["Shows the header of the Achievementtracker"],
						get	=	function(info) return aat.db.profile.ShowHeader end,
						set	=	function(info, newval)
										aat.db.profile.ShowHeader	=	newval
										AchievementWatchAnchor.UpdateFrame = nil
									end,
						hidden	= function() return not aat:IsEnabled() end,
						},--[[
					savelast	=	{
						type	=	'toggle',
						order	=	6,
						width	=	'full',
						name	=	L["Save tracked Achievement"],
						desc	=	L["Save last tracked Achievement and resore it after login"],
						get	=	function(info) return aat.db.profile.savelast end,
						set	=	function(info, newval)
										aat.db.profile.savelast	=	newval
									end,
						hidden	= function() return not aat:IsEnabled() end,
					},--]]
					RemoveFinished	=	{
						type	=	'toggle',
						order	=	7,
						width	=	'full',
						name	=	L["Remove Completed Achievement"],
						desc	=	L["Automatical remove the completed Achievement from tracker"],
						get	=	function(info) return aat.db.profile.RemoveFinished end,
						set	=	function(info, newval)
									aat.db.profile.RemoveFinished	=	newval
								end,
						hidden	= function() return not aat:IsEnabled() end,
						},--[[
					Spacing2	=	{
						type	=	'header',
						order	=	7,
						name	=	'',
					},--]]
					ManageLines	=	{
						type	=	'toggle',
						order	=	8,
		--							width	=	'full',
						name	=	L["Manage Achievement Lines"],
						desc	=	L["Mange count of Lines to optimal display objectives from the achievement"],
						get	=	function(info) return aat.db.profile.ManageLines end,
						set	=	function(info, newval)
								aat.db.profile.ManageLines = newval
								WatchFrame_Update()
							end,
						hidden	= function() return not aat:IsEnabled() end,
					},
					MaxLines	=	{
						type	=	'range',
						order	=	9,
		--							width	=	'full',
						name	=	L["Number of Lines"],
						desc	=	L["Number of lines that can be used by the Linemanger"],
						get	=	function(info) return aat.db.profile.NumLines end,
						set	=	function(info, newval)
									aat.db.profile.NumLines	=	newval
									aat:AchievementTracker_DisplayAchievements()
								end,
						min	=	1,
						max	=	100,
						step	=	1,
						disabled	=	function(info) return not aat.db.profile.ManageLines end,
						hidden	= function() return not aat:IsEnabled() end,
					},
					Spacing3	=	{
						type	=	'header',
						order	=	10,
						name	=	'',
						hidden	= function() return not aat:IsEnabled() end,
					},
					ATrackerFontSize	=	{
						type	=	'range',
						order	=	11,
						width	=	'full',
						name	=	L["Achievement Tracker Scale"] .. "\n(delete me)",
						desc	=	L["Sets the Scale of the Achievement Tracker"],
						get	=	function(info) return aat.db.profile.FontSize end,
						set	=	function(info, newval)
									aat.db.profile.FontSize	=	newval;
									AchievementWatchAnchor.UpdateFrame = nil;
								end,
						min	=	8,
						max	=	18,
						step	=	1,
						hidden	= function() return not aat:IsEnabled() end,
					},
					MouseEvents = {
						type	=	'group',
						guiInline	=	true,
						name	=	L["Mouse Events"],
						order	=	12,
						args = {
							ShowTooltip = {
								type = 'toggle',
								order = 1,
								name = L['Show tooltip'],
								desc = L['Show tooltip on achievemet tracker'],
								get = function(info) return aat.db.profile.showtooltip end,
								set = function(info, newval) aat.db.profile.showtooltip = newval; AchievementWatchAnchor.UpdateFrame = nil; end,
							},
							ShowTooltipHint = {
								type = 'toggle',
								order = 2,
								name = L['Show hint tooltip'],
								desc = L['Show tooltip hint in the achievemet tracker tooltip'],
								get = function(info) return aat.db.profile.showtooltiphint end,
								set = function(info, newval) aat.db.profile.showtooltiphint = newval end,
							},
							MouseLeftClick = {
								type = 'toggle',
								order = 3,
								name = L["Enable Left Click"],
								desc = L['Enable Left+Click to open achievement'],
								get = function(info) return aat.db.profile.clickleft end,
								set = function(info, newval) aat.db.profile.clickleft = newval AchievementWatchAnchor.UpdateFrame = nil end,
							},
							MouseRightClick = {
								type = 'toggle',
								order = 4,
								name = L["Enable Right Click"],
								desc = L['Enables Right-Click on the achievementtracker to remove a tracked acheivement'],
								get = function(info) return aat.db.profile.clickright end,
								set = function(info, newval) aat.db.profile.clickright = newval AchievementWatchAnchor.UpdateFrame = nil end,
							},
						},
						hidden	= function() return not aat:IsEnabled() end,
					},

		-- Colors on Achievementtracker

					Colors	=	{
						type	=	'group',
						guiInline	=	true,
						name	=	L["Colors"]..'',
						desc	=	L["Set tracker Colors"],
						order	=	14,
						args	=	{
							Background	=	{
								type	=	'toggle',
								order	=	1,
								--width	=	'full',
								name	=	L["Background"],
								desc	=	L["Use Background"],
								get	=	function(info) return aat.db.profile.showbg end,
								set	=	function(info, newval)
											aat.db.profile.showbg	=	newval
											AchievementWatchAnchor.UpdateFrame = nil
										end,
							},
							CustomBackground	=	{
								type	=	'toggle',
								order	=	2,
								--width	=	'full',
								name	=	L["Custom Background Color"],
								desc	=	L["Use custom color for background"],
								get	=	function(info) return aat.db.profile.ownbgcolor end,
								set	=	function(info, newval)
											aat.db.profile.ownbgcolor	=	newval
											AchievementWatchAnchor.UpdateFrame = nil
										end,
								disabled	=	function(info) return not aat.db.profile.showbg end,
							},
							BackgroundColor	=	{
								type	=	'color',
								order	=	3,
								name	=	L["Background Color"],
								desc	=	L["Sets the Background Color"],
								get	=	function(info)
											local a	=	aat.db.profile.Colors.BgColor.a
											local b	=	aat.db.profile.Colors.BgColor.b
											local g	=	aat.db.profile.Colors.BgColor.g
											local r	=	aat.db.profile.Colors.BgColor.r
											return r, g, b, a
								end,
								set	=	function(info, r, g, b, a)
												aat.db.profile.Colors.BgColor	=	{
																r	=	r,
																g	=	g,
																b	=	b,
																a	=	a,
												}
											AchievementWatchAnchor.UpdateFrame = nil
								end,
								hasAlpha	=	true,
								disabled	=	function(info) return not aat.db.profile.ownbgcolor end,
							},
							QuestTracker_BgBorder	=	{
								type	=	'color',
								order	=	4,
								name	=	L["Background Corner Color"],
								desc	=	L["Sets the Background Corner Color"],
								get	=	function(info)
											local a	=	aat.db.profile.Colors.BgCornerColor.a
											local b	=	aat.db.profile.Colors.BgCornerColor.b
											local g	=	aat.db.profile.Colors.BgCornerColor.g
											local r	=	aat.db.profile.Colors.BgCornerColor.r

											--return , , ,
											return r, g, b, a
								end,
								set	=	function(info, r, g, b, a)
												aat.db.profile.Colors.BgCornerColor	=	{
																r	=	r,
																g	=	g,
																b	=	b,
																a	=	a,
												}
											AchievementWatchAnchor.UpdateFrame = nil
								end,
								hasAlpha	=	true,
								disabled	=	function(info) return not aat.db.profile.ownbgcolor end,
							},
							ATrackerAlpha	=	{
								type	=	'range',
								order	=	5,
								width	=	'full',
								name	=	L["Achievement Tracker Alpha"],
								desc	=	L["Sets the Alpha of the Achievement Tracker"],
								get	=	function(info) return aat.db.profile.alpha end,
								set	=	function(info, newval)
											aat.db.profile.alpha	=	newval
											AchievementWatchAnchor.UpdateFrame = nil
										end,
								min	=	0,
								max	=	1,
								step	=	0.01,
							},
						},
						hidden	= function() return not aat:IsEnabled() end,
					},

				},
			},
		},
	}

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(AddonName, MainOptionsTable, AddonName)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName, nil, nil, "general")

end

function aat:AnchorFrame_OnLoad(frame)
	AchievementWatchAnchor = frame
end

function aat:TrackerFrame_OnLoad(frame)
	AchievementWatchFrame = frame
end

--
--- Timed Frame Events
--

function aat:WatchFrameLine_OnUpdate(frame, elapsed)
	if frame:IsMouseOver() then
		local line = frame:GetParent()
		if GetMouseFocus() == AchievementWatchFrame then
			AchievementWatchFrame.aID = line.aID
			AchievementWatchFrame.subaID = line.subaID
			if frame.IsIcon then
				AchievementWatchFrame.IsIcon = true
			else
				AchievementWatchFrame.IsIcon = nil
			end
			AchievementWatchFrame.lastIdUpdate = GetTime() + .2
		end
	end
end

function aat:AnchorFrame_OnUpdate(frame, elapsed)
	if frame.UIScale ~= UIParent:GetScale() and aat and aat.db then

		local posx = aat.db.char.FramePosition.posx
		local posy = aat.db.char.FramePosition.posy
		local point = aat.db.char.FramePosition.point

		frame:ClearAllPoints()
		frame:SetPoint("TOPLEFT", UIParent, point, posx/UIParent:GetScale(), posy/UIParent:GetScale())
		frame.UIScale = UIParent:GetScale();
--		print(UIParent:GetWidth() .. " - " .. UIParent:GetHeight() .. " - " .. UIParent:GetScale())
	end
--	print(frame.UpdateFrame)
	if frame.UpdateFrame == nil then
		frame.UpdateFrame = aat:AchievementTracker_UpdateFrame()
	end
end

function aat:AchievementTracker_OnUpdate(frame, elapsed)
	if not frame.aID then return end
	
	if (frame.lastIdUpdate or 0 ) < (GetTime()) then
		frame.aID = nil;
		frame.subaID = nil;
		frame.IsIcon = nil;
	end
	
	-- hide tooltip when refreshtime is over or mainframe lost mousefocus
	if not frame.aID or GetMouseFocus() ~= frame then
		if frame.Tooltip:IsVisible() then
			frame.Tooltip:FadeOut()
			frame.Tooltip.aID = nil;
			frame.Tooltip.IsIcon = nil;
			ResetCursor()
		end
	elseif frame.aID and GetMouseFocus() == frame then
		if ((frame.subaID and frame.subaID) or frame.aID) ~= frame.Tooltip.aID or frame.IsIcon ~= frame.Tooltip.IsIcon then

		-- show tooltip
		ResetCursor()
		ShowInspectCursor()
		frame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")

		local _, achievementName, _, completed, _, _, _, description = GetAchievementInfo((frame.subaID and frame.subaID) or frame.aID);
		frame.Tooltip:AddLine(achievementName)
		frame.Tooltip:AddLine(" ")
		frame.Tooltip:AddLine(description,1,1,1,true)

		local tooltiphint

		if aat.db.profile.showtooltiphint then

			if aat.db.profile.clickleft then
				tooltiphint = L['|cffeda55fClick|r to open achievement, |cffeda55fShift+Click|r to link achievement to chat.']
			end

			if aat.db.profile.clickright and not frame.IsIcon then
				tooltiphint = tooltiphint and tooltiphint .. L['|cffeda55fRight-Click|r to cancel tracking of this achievement'] or L['|cffeda55fRight-Click|r to cancel tracking of this achievement']
			elseif aat.db.profile.clickright and frame.IsIcon then
				tooltiphint = tooltiphint and tooltiphint .. L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"] or L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"]
			end

			if tooltiphint then
				frame.Tooltip:AddLine(tooltiphint,0,1,0,true)
			end
		end

		if aat.db.profile.debug then
			frame.Tooltip:AddLine("AchievementID: " .. frame.aID,nil,nil,nil,true)
		end

		frame.Tooltip:Show()
		
		frame.Tooltip.aID = (frame.subaID and frame.subaID) or frame.aID
		frame.Tooltip.IsIcon = frame.IsIcon
		
		end
		
		
	end
		
	
end


--  Initial functions

function aat:AchievementTracker_OnMouseUp(frame, button)
	if aat.db.char.AcheivementWatchMinimized then
		return;
	end
	if button == "LeftButton" and (GetTime() - frame.leftclicktime) < 1 then
		aat:AchievementTracker_Click(frame, button)
	elseif button == "RightButton" and (GetTime() - frame.rightclicktime) < 1 then
		aat:AchievementTracker_Click(frame, button)
	end
end

function aat:AchievementTracker_Minimize(button)
	button = button or aatAchievementWatchFrame.TitleFrame.Minimize
	aat.db.char.AchievementWatchMinimized = true;

	button:SetNormalTexture("Interface\\AddOns\\"..AddonName.."\\Images\\restore_up")
	button:SetPushedTexture("Interface\\AddOns\\"..AddonName.."\\Images\\restore_down")

	aatAchievementWatchFrame:SetHeight(1)
	aatAchievementWatchFrame:SetWidth(aatAchievementWatchFrame.TitleFrame.Text:GetWidth()+32)
	aatAchievementWatchFrame.Backdrop:Hide()
	aatAchievementWatchFrame.LineFrame:Hide()
end

function aat:AchievementTracker_Maximize(button)
	button = button or aatAchievementWatchFrame.TitleFrame.Minimize
	aat.db.char.AchievementWatchMinimized = false;

	button:SetNormalTexture("Interface\\AddOns\\"..AddonName.."\\Images\\minimize_up")
	button:SetPushedTexture("Interface\\AddOns\\"..AddonName.."\\Images\\minimize_down")

	aatAchievementWatchFrame.Backdrop:Show()
	aatAchievementWatchFrame.LineFrame:Show()

	aat:AchievementTracker_DisplayAchievements()
end

--
--  EVENTS
--

function aat:Event_TRACKED_ACHIEVEMENT_UPDATE(...)
	print(...)
	if not aat:IsEnabled() then
		return;
	end
	aat:AchievementTracker_DisplayAchievements()
end

-- Remove tracked achievement
function aat:Event_ACHIEVEMENT_EARNED ()
	if not aat:IsEnabled() then
		return;
	end
	local lastfinished = GetLatestCompletedAchievements()
	if aat:AchievementTracker_IsTracked(lastfinished) then
		RemoveTrackedAchievement(lastfinished)
	end
end


-- Addon funktions

function aat:AchievementTracker_IsTracked(id)
	local Achievements = { GetTrackedAchievements() }

	for k, aID in ipairs(Achievements) do
		if id == aID then
			return true
		end
	end
	return false
end

--[[
--------------------------------------------
-- Notes:
-- Update base stats of Achievementtracker
--
-- Returns:
-- * true for updated or nil
--------------------------------------------]]
function aat:AchievementTracker_UpdateFrame()
	local aatAchievementWatchFrame = aatAchievementWatchFrame
	if not aat or not aat.db then
		return
	end
	
	if aat.db.profile.Locked then
		aatAchievementWatchAnchor:SetMovable(nil);
	else
		aatAchievementWatchAnchor:SetMovable(true);
	end

	-- title
	local title = aatAchievementWatchFrame.TitleFrame
	if aat.db.profile.ShowHeader then
		title:Show();
		local t1, _, t2 = aatAchievementWatchFrame.TitleFrame.Text:GetFont();
		aatAchievementWatchFrame.TitleFrame.Text:SetFont(t1, aat.db.profile.FontSize + 2 , t2);
		aatAchievementWatchFrame.TitleFrame:SetHeight(aat.db.profile.FontSize + 18)
	else
		title:Hide();
		if aat.db.char.AchievementWatchMinimized then
			aat:AchievementTracker_Maximize();
			aat.db.char.AchievementWatchMinimized = false;
		end
	end
	
	
	
	if aat.db.profile.Locked then
		title:EnableMouse(nil);
		title:RegisterForDrag(nil);
	else
		title:EnableMouse(true);
		title:RegisterForDrag("LeftButton");
	end
	if aat.db.profile.showbg then
		if not aat.db.profile.ownbgcolor then
		-- Set Tooltipcolor as BG color
			title:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
			title:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		else
		-- Set own Colors
			title:SetBackdropColor(aat.db.profile.Colors.BgColor.r, aat.db.profile.Colors.BgColor.g, aat.db.profile.Colors.BgColor.b, aat.db.profile.Colors.BgColor.a)
			title:SetBackdropBorderColor(aat.db.profile.Colors.BgCornerColor.r, aat.db.profile.Colors.BgCornerColor.g, aat.db.profile.Colors.BgCornerColor.b, aat.db.profile.Colors.BgCornerColor.a)
		end
	else
		title:SetBackdropColor(0,0,0,0)
		title:SetBackdropBorderColor(0,0,0,0)
	end

	
	if aat.db.profile.Locked then
		aatAchievementWatchFrame:EnableMouse(true);
		aatAchievementWatchFrame:RegisterForDrag(nil);
	else
		aatAchievementWatchFrame:EnableMouse(true);
		aatAchievementWatchFrame:RegisterForDrag("LeftButton");
	end
	
	if aat.db.profile.showbg then
		if not aat.db.profile.ownbgcolor then
		-- Set Tooltipcolor as BG color
			aatAchievementWatchFrame.Backdrop:SetBackdropBorderColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
			aatAchievementWatchFrame.Backdrop:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
		else
		-- Set own Colors
			aatAchievementWatchFrame.Backdrop:SetBackdropColor(aat.db.profile.Colors.BgColor.r, aat.db.profile.Colors.BgColor.g, aat.db.profile.Colors.BgColor.b, aat.db.profile.Colors.BgColor.a)
			aatAchievementWatchFrame.Backdrop:SetBackdropBorderColor(aat.db.profile.Colors.BgCornerColor.r, aat.db.profile.Colors.BgCornerColor.g, aat.db.profile.Colors.BgCornerColor.b, aat.db.profile.Colors.BgCornerColor.a)
		end
	else
		aatAchievementWatchFrame.Backdrop:SetBackdropColor(0,0,0,0)
		aatAchievementWatchFrame.Backdrop:SetBackdropBorderColor(0,0,0,0)
	end

	if not aat.db.profile.showtooltip and aat.db.profile.Locked and not aat.db.profile.clickleft and not aat.db.profile.clickright then
		aatAchievementWatchFrame:EnableMouse(nil);
	end
	if aat.db.profile.Locked and not aat.db.profile.clickleft and not aat.db.profile.clickright then
		aatAchievementWatchFrame:EnableMouse(nil);
	end
	
	-- Alpha
	aatAchievementWatchFrame:SetAlpha(aat.db.profile.alpha)
	
	if aatAchievementWatchFrame:IsShown() then
		aat:AchievementTracker_DisplayAchievements()
	end

	-- calc out font size and spaces
	if aat.db.profile.FontSize ~= NormalFontSize then
		aat:AchievementTracker_CalcFonts()
	end
	return true
end

function aat:AchievementTracker_CalcFonts()
	--[[local NormalFontSize, HeaderFontSize, TitleFontSize, FontString
	local BorderSpacing, NormalFontSpacing, HeaderFontSpacing
	local IconSizeHeader, IconSizeObjective
	local AchievementWatchFrame, AchievementWatchAnchor]]

	local line = FrameCache:GetFrame();
	FontString = line.text:GetFont();
	line.frameCache:ReleaseFrame(line);
	
	-- font size
	NormalFontSize = aat.db.profile.FontSize
	TitleFontSize = NormalFontSize * 1.25
	HeaderFontSize = NormalFontSize * 1.10
	
	-- font spacing
	BorderSpacing = TitleFontSize/3
	NormalFontSpacing = NormalFontSize/6
	HeaderFontSpacing = HeaderFontSize/3
	
	IconSizeHeader = HeaderFontSize + HeaderFontSpacing-- + HeaderFontSpacing
	IconSizeObjective = NormalFontSize + NormalFontSpacing-- + NormalFontSpacing
end

function aat:AchievementTracker_ResetLine(line)
	-- reset frames
	line:ClearAllPoints();
	
	line.text:SetText("");
	line.text:SetTextColor(0.8, 0.8, 0.8);
	line.text:SetFont(FontString,NormalFontSize)
	line.text:SetWidth(0)
	line.text:SetHeight(0);	
	line.text:Show();
	
	line.dash:SetText(nil);
	line.dash:SetTextColor(0.8, 0.8, 0.8);
	line.dash:SetFont(FontString,NormalFontSize)
	line.dash:SetWidth(0);
	line.dash:SetHeight(0);	
	line.dash:Show();

	line.mouseover:SetFrameLevel(1);
	line.mouseover:ClearAllPoints();
	line.mouseover:SetPoint("TOPLEFT", line.text)
	line.mouseover:SetPoint("BOTTOMLEFT", line.text)
	
	line.statusBar:Hide();
	line.iconframe:Hide();
	line:SetHeight(NormalFontSize + (2*NormalFontSpacing));

	line.iconframe.aID = nil;
	line.criteriaID = nil;	

	-- mod start
	line.isHeader = nil;
	line.type = nil;
	line.qID = nil;
	line.aID = nil;
	line.subaID = nil;
	line.cType = nil
	line.hasItem = nil
end

function aat:AchievementTracker_GetLine ()

	local line = AchievementLines[AchievementLineIndex];
	if ( not line ) then
		AchievementLines[AchievementLineIndex] = FrameCache:GetFrame();
		line = AchievementLines[AchievementLineIndex];
	end

	line:Reset();


	line:SetParent(AchievementWatchFrame.LineFrame)
	AchievementLineIndex = AchievementLineIndex + 1;
	return line;
end

function aat:AchievementTracker_ReleaseUnusedLines ()
	local line
	for i = AchievementLineIndex, #AchievementLines do
		line = AchievementLines[i];
		line:Hide();
		line.frameCache:ReleaseFrame(line);
		AchievementLines[i] = nil;
	end

	AchievementLineIndex = 1
end

function aat:AchievementTracker_UpdateFrameStyle()
	local FrameWidth = 0
	AchievementWatchFrame.TitleFrame.Text:SetFont(FontString,TitleFontSize)
	AchievementWatchFrame.TitleFrame:SetHeight(TitleFontSize * 2)
	
	FrameWidth = AchievementWatchFrame.TitleFrame.Text:GetStringWidth() + --[[(2 * BorderSpacing) +--]] AchievementWatchFrame.TitleFrame.Minimize:GetWidth()
	
	if aat.db.char.AchievementWatchMinimized then
		AchievementWatchFrame:SetWidth(FrameWidth)
		AchievementWatchFrame:SetHeight(0.1)
		return
	end
	
	AchievementWatchFrame.TitleFrame.Text:ClearAllPoints()
	AchievementWatchFrame.TitleFrame.Text:SetPoint("LEFT", AchievementWatchFrame.TitleFrame, "LEFT", BorderSpacing,0)
	--FrameWidth = AchievementWatchFrame.TitleFrame.Text:SetPoint()
	--AchievementWatchFrame.TitleFrame:SetWidth(FrameWidth)

	local line
	
	for i=1, #AchievementLines do
		line = AchievementLines[i]
--		line:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"});
--		line.mouseover:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background"});
		if ceil(i/2) == floor(i/2) then
			line:SetBackdropColor(0.25, 0.75, 0.25,0.25)
		else
			line:SetBackdropColor(1, 1, 0,0.25)
		end
		line.mouseover:SetBackdropColor(1, 0.5, 0.25,1)
		
		if line.aType == "header" then
			line.text:SetFont(FontString, HeaderFontSize)
			line.dash:SetFont(FontString, HeaderFontSize)
			line:SetHeight(HeaderFontSize + (2*HeaderFontSpacing))
			line.dash:SetPoint("TOPLEFT", line, "TOPLEFT", HeaderFontSpacing,-HeaderFontSpacing)
			line.mouseover:SetWidth(line.dash:GetStringWidth() + line.text:GetStringWidth())
			if line.iconframe:IsShown() then
				line.iconframe:ClearAllPoints()
				line.iconframe:SetPoint("LEFT", line, "LEFT",HeaderFontSpacing,0)
				line.iconframe:SetHeight(IconSizeHeader)
				line.iconframe:SetWidth(IconSizeHeader)
				line.icon:SetHeight(IconSizeHeader)
				line.icon:SetWidth(IconSizeHeader)
				local offset = IconSizeHeader/3.2
				line.iconborder:ClearAllPoints()
				line.iconborder:SetPoint("TOPLEFT", line.icon, "TOPLEFT", -offset, offset)
				line.iconborder:SetPoint("BOTTOMRIGHT", line.icon, "BOTTOMRIGHT", offset, -offset)
				line.dash:SetPoint("TOPLEFT",line, "TOPLEFT", IconSizeHeader + (2*HeaderFontSpacing),-HeaderFontSpacing)
				FrameWidth = max(FrameWidth, line.dash:GetStringWidth() + line.text:GetStringWidth() + (2*HeaderFontSpacing) + IconSizeHeader)
			end
		else
			line.text:SetFont(FontString, NormalFontSize)
			line.dash:SetFont(FontString, NormalFontSize)
			line.dash:SetPoint("TOPLEFT", line, "TOPLEFT", 0,-NormalFontSpacing)
			line:SetHeight(NormalFontSize + (2*NormalFontSpacing))
		end
		
		if line.aType == "sBar" then
			-- StatusBar
			line.statusBar:ClearAllPoints()
			line.statusBar:SetPoint("TOPLEFT",line,"TOPLEFT",3*HeaderFontSpacing,-HeaderFontSpacing)
			line.statusBar:SetPoint("BOTTOMRIGHT",line,"BOTTOMRIGHT",-(3*HeaderFontSpacing),HeaderFontSpacing)
			line:SetHeight(2*NormalFontSize)
			line.statusBar.text:SetFont(FontString, NormalFontSize*0.8)
			line.mouseover:SetAllPoints(line.statusBar)
			FrameWidth = max(FrameWidth, 200)
		elseif line.aType == "alink" then
			-- Objective is a Link to an other Achievement
			local textwidth = line.dash:GetStringWidth() + line.text:GetStringWidth()
			line.mouseover:SetWidth(textwidth)
			if line.iconframe:IsShown() then
				line.iconframe:ClearAllPoints()
				line.iconframe:SetPoint("LEFT", line, "LEFT",(2*NormalFontSpacing) + textwidth,0)
				line.iconframe:SetHeight(IconSizeObjective)
				line.iconframe:SetWidth(IconSizeObjective)
				line.icon:SetHeight(IconSizeObjective)
				line.icon:SetWidth(IconSizeObjective)
				local offset = IconSizeObjective/3.2
				line.iconborder:ClearAllPoints()
				line.iconborder:SetPoint("TOPLEFT", line.icon, "TOPLEFT", -offset, offset)
				line.iconborder:SetPoint("BOTTOMRIGHT", line.icon, "BOTTOMRIGHT", offset, -offset)
				--line.dash:SetPoint("TOPLEFT",line, "TOPLEFT", 0,-HeaderFontSpacing)
				FrameWidth = max(FrameWidth, textwidth + (3*NormalFontSpacing) + IconSizeObjective)
			else
				FrameWidth = max(FrameWidth, textwidth)
			end
		elseif line.aType == "normal" then
			-- Normal Objective
			local textwidth = line.dash:GetStringWidth() + line.text:GetStringWidth()
			line.mouseover:SetWidth(textwidth)
			FrameWidth = max(FrameWidth, textwidth)
		elseif line.aType == "disc" then
			-- Achievement with no Objective, use Inscrition
			--line.text:SetPoint("TOPRIGHT", line, "TOPRIGHT",0,-NormalFontSpacing)
			line.mouseover:SetAllPoints(line.text)
			FrameWidth = max(FrameWidth, 200)
		elseif line.aType == "cont" then
			-- the "..." line
			line.mouseover:SetAllPoints(line.text)
			FrameWidth = max(FrameWidth, line.dash:GetStringWidth() + line.text:GetStringWidth())
		else
			-- do nothing
		end
	end
	
	local FrameHeight = - NormalFontSize
	for i=1, #AchievementLines do
		line = AchievementLines[i]
		
		if line.aType == "disc" then
			line.text:SetWidth(FrameWidth - line.dash:GetStringWidth())
			line:SetHeight(line.text:GetStringHeight() + (2*NormalFontSpacing))
			FrameHeight = FrameHeight + line.text:GetStringHeight() + (2*NormalFontSpacing)
		elseif line.aType == "header" then
			FrameHeight = FrameHeight + (2*HeaderFontSpacing) + HeaderFontSize + NormalFontSize
		elseif line.aType =="sBar" then
			FrameHeight = FrameHeight + (2*NormalFontSize)
		else
			FrameHeight = FrameHeight + (2*NormalFontSpacing) + NormalFontSize
		end
	end
	
	AchievementWatchFrame:SetWidth(FrameWidth)
	AchievementWatchFrame:SetHeight(FrameHeight)
end

function aat:AchievementTracker_DisplayAchievements()

	if aat.db.char.AchievementWatchMinimized then
		-- tracker minimized, exit
		if aat.db.profile.ShowHeader then
			aatAchievementWatchFrame:Show();
			aatAchievementWatchFrame.TitleFrame.Text:SetText(L["Achievement Tracker"] .. " ("..select("#", GetTrackedAchievements()) .. "/" .. WATCHFRAME_MAXACHIEVEMENTS .. ")")
		else
			aatAchievementWatchFrame:Hide();
		end
		return
	end

	local _; -- Doing this here thanks to IBLJerry!
	local TrackedAchievements = {GetTrackedAchievements()}
	local numTrackedAchievements = #TrackedAchievements

	--HACK:
	lineFrame = aatAchievementWatchFrame.LineFrame
	initialOffset = 0
	local WATCHFRAME_CRITERIA_PER_ACHIEVEMENT = WATCHFRAME_CRITERIA_PER_ACHIEVEMENT + 1; -- + 1 iss a dummy, because we handle the latst line in an other way
	local Achievements = {}


	local achievementID;
	local achievementName, completed, description, icon;

	-- line variables
	local line, achievementTitle, previousLine;

	local numCriteria, criteriaDisplayed;
	local criteriaString, criteriaType, criteriaCompleted, quantity, totalQuantity, name, flags, assetID, quantityString, criteriaID, achievementCategory;

	-- get used lines per achievement
	local maxlines = 0
	local extralines = 0;
	local usedlines = 0
	local achievementusedlines
	for i = 1 , numTrackedAchievements, 1 do
		achievementID = TrackedAchievements[i] --select(i, GetTrackedAchievements());
		Achievements[achievementID] = {}
		Achievements[achievementID].criteria = {}
		numCriteria = GetAchievementNumCriteria(achievementID)
		usedlines = usedlines + 1 -- title
		achievementusedlines = 0
		for j=1, numCriteria, 1 do
			if not select(3, GetAchievementCriteriaInfo(achievementID, j)) then
				usedlines = usedlines + 1
				achievementusedlines = achievementusedlines + 1
				table.insert(Achievements[achievementID].criteria, j)
			end
			if numCriteria == 1 and ( bit.band(select(7, GetAchievementCriteriaInfo(achievementID, j)), ACHIEVEMENT_CRITERIA_PROGRESS_BAR) == ACHIEVEMENT_CRITERIA_PROGRESS_BAR ) then
				extralines = extralines + 1
			end
		end

		if numCriteria == 0 then
			-- Achievement with no objective, we need 1 line for discription
			usedlines = usedlines + 1
		end
		if achievementusedlines > maxlines then
			maxlines = achievementusedlines
		end
		Achievements[achievementID].NumLines = achievementusedlines;
	end

	-- calc optimal lines per achievement
	if aat.db.profile.ManageLines and type(aat.db.profile.NumLines) == 'number' then
		WATCHFRAME_CRITERIA_PER_ACHIEVEMENT = 0
		for i = maxlines, 1, -1 do
			usedlines = 0
			for k in pairs(Achievements) do
				if Achievements[k].NumLines > i then
					usedlines = usedlines + i
				else
					usedlines = usedlines + Achievements[k].NumLines
				end

				if Achievements[k].NumLines == 0 then
					-- Achievement with no objective, we need 1 line for discription
					usedlines = usedlines + 1
				end
			end

			if (usedlines + numTrackedAchievements) <= aat.db.profile.NumLines then
				aat:debugprint("break at " .. i .. " Line(s) per Achievement, " .. (usedlines + numTrackedAchievements) .. " Lines total, " .. (aat.db.profile.NumLines - (usedlines + numTrackedAchievements)).. " Lines in reserve")
				extralines = aat.db.profile.NumLines - (usedlines + numTrackedAchievements)
				WATCHFRAME_CRITERIA_PER_ACHIEVEMENT = i
				break
			end
		end
	end

	local displayOnlyArena = (not WatchFrame:IsUserPlaced()) and ArenaEnemyFrames and ArenaEnemyFrames:IsShown();
	for i = 1, numTrackedAchievements do
		achievementID = TrackedAchievements[i] --select(i, GetTrackedAchievements());
		achievementCategory = GetAchievementCategory(achievementID);
		if ( (not displayOnlyArena) or achievementCategory == WATCHFRAME_ACHIEVEMENT_ARENA_CATEGORY ) then
			_, achievementName, _, completed, _, _, _, description, _, icon = GetAchievementInfo(achievementID);

			--
			-- Title
			--

			line = aat:AchievementTracker_GetLine(1);
			line.aType = "header";
			line.aID = achievementID
			achievementTitle = line;
			line.text:SetText(achievementName);
			line.mouseover:SetWidth(line.text:GetStringWidth())
			if ( completed ) then
				line.text:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			else
				line.text:SetTextColor(0.75, 0.61, 0);
			end
			line:Show()
			local lineWidth = line.text:GetStringWidth()
			if ( previousLine ) then -- If this isn't the first displayed title, our position is relative to the last displayed line.
				line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, -NormalFontSize);
				line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, -NormalFontSize);
			else
				line:SetPoint("TOPRIGHT", lineFrame, "TOPRIGHT", 0, initialOffset);
				line:SetPoint("TOPLEFT", lineFrame, "TOPLEFT", 0, initialOffset);
			end
			
			if icon then
				line.icon:SetTexture(icon);
				line.iconframe:Show()
			end

			previousLine = line;

			--
			-- Details (Criteria)
			--

			numCriteria = Achievements[achievementID].NumLines;
			if numCriteria > WATCHFRAME_CRITERIA_PER_ACHIEVEMENT then
				if extralines > 0 then
					-- we have empty lines over, so we use them
					numCriteria = WATCHFRAME_CRITERIA_PER_ACHIEVEMENT + 1
					extralines = extralines - 1
				else
					numCriteria = WATCHFRAME_CRITERIA_PER_ACHIEVEMENT
				end
			end

			if ( numCriteria > 0 ) then

				--
				-- Achivements with Criteria
				--
				criteriaDisplayed = 0;
				for j = 1, numCriteria do
					criteriaString, criteriaType, criteriaCompleted, quantity, totalQuantity, name, flags, assetID, quantityString, criteriaID = GetAchievementCriteriaInfo(achievementID, Achievements[achievementID].criteria[j]);
					if ( j == numCriteria and (numCriteria < Achievements[achievementID].NumLines) ) then
						-- We ran out of space to display incomplete criteria >_<
						line = aat:AchievementTracker_GetLine();
						line.aType = "cont";
						line.aID = achievementID
						line.dash:SetText("   ");
						line.text:SetText("...");
						line.text:SetTextColor(0.8, 0.8, 0.8);
						line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0 , 0);
						line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
						line:Show();
						criteriaDisplayed = criteriaDisplayed + 1;
						previousLine = line;
					else

						--
						-- Timed Criteria
						--
						if ( WATCHFRAME_TIMEDCRITERIA[criteriaID] ) then
							local timedCriteria = WATCHFRAME_TIMEDCRITERIA[criteriaID]
							line = aat:AchievementTracker_GetLine();
							line.criteriaID = criteriaID;
							line.duration = timedCriteria.duration;
							line.startTime = timedCriteria.startTime;

							line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
							line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, 0);
							line:Show()
							criteriaDisplayed = criteriaDisplayed + 1;
							previousLine = line;
							WatchFrameLines_AddUpdateFunction(WatchFrame_UpdateTimedAchievements);
						end

						--
						-- Criteria with Statusbar
						--
						if ( bit.band(flags, ACHIEVEMENT_CRITERIA_PROGRESS_BAR) == ACHIEVEMENT_CRITERIA_PROGRESS_BAR ) then
							line = aat:AchievementTracker_GetLine();
							line.aType = "sBar"
							line.aID = achievementID
							line.statusBar:Show();
							line.statusBar:GetStatusBarTexture():SetVertexColor(0, 0.6, 0, 1);
							line.statusBar:SetMinMaxValues(0, totalQuantity);
							line.statusBar:SetValue(quantity);
							line.statusBar.text:SetText(quantityString);
							line.mouseover:ClearAllPoints()
							line.mouseover:SetAllPoints(line.statusBar)

							line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
							line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, 0);
							line:Show();
							criteriaDisplayed = criteriaDisplayed + 2;
							previousLine = line;
						else

							--
							-- Normal Achievements with objectives
							--
							line = aat:AchievementTracker_GetLine();
							line.aID = achievementID
							line.text:SetText(" - " .. criteriaString);
							line.text:SetTextColor(0.8, 0.8, 0.8);

							if criteriaType == 8 then
								line.aType = "alink"
								line.subaID = assetID
								line.iconframe:Show()
								line.icon:SetTexture(select(10, GetAchievementInfo(assetID)))
								line.iconframe.aID = assetID
							else
								line.aType = "normal"
							end
							line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
							line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, 0);
							line:Show();
							criteriaDisplayed = criteriaDisplayed + 1;
							previousLine = line;
						end
					end
				end
			else

				--
				-- Achievement with no criteria, display Discription
				--
				line = aat:AchievementTracker_GetLine();
				line.dash:SetText(" - ");
				line.dash:SetTextColor(0.8, 0.8, 0.8);

				local dashWidth = line.dash:GetStringWidth();

				line.aID = achievementID
				line.text:SetText(description);
				line.mouseover:ClearAllPoints()
				line.mouseover:SetAllPoints(line.text)
				line.text:SetTextColor(0.8, 0.8, 0.8);
				line.aType = "disc";

				line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
				line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, 0);
				line:Show();

				previousLine = line;

				for criteriaID, timedCriteria in next, WATCHFRAME_TIMEDCRITERIA do
					if ( timedCriteria.achievementID == achievementID ) then
						line = aat:AchievementTracker_GetLine();
						line.criteriaID = criteriaID;
						line.duration = timedCriteria.duration;
						line.startTime = timedCriteria.startTime;

						line:SetPoint("TOPRIGHT", previousLine, "BOTTOMRIGHT", 0, 0);
						line:SetPoint("TOPLEFT", previousLine, "BOTTOMLEFT", 0, 0);
						line:Show()
						previousLine = line;
						WatchFrameLines_AddUpdateFunction(WatchFrame_UpdateTimedAchievements);
					end
				end
			end
		end
	end

	
	
	aat:AchievementTracker_ReleaseUnusedLines();

	if previousLine then
		aatAchievementWatchFrame:Show()
	else
		aatAchievementWatchFrame:Hide()
		return;
	end


	-- titlebar
	aatAchievementWatchFrame.TitleFrame.Text:SetText(L["Achievement Tracker"] .. " ("..numTrackedAchievements .. "/" .. WATCHFRAME_MAXACHIEVEMENTS .. ")")
end

function aat:AchievementTracker_Click(frame, button)
	-- check for valid AchievementID
	if (not frame.aID or frame.aID <= 0) or aat.db.char.AchievementWatchMinimized then
		-- invalid AchievementID
		return
	end

	-- Handle Icon click
	if frame.IsIcon and frame.subaID and button == "RightButton" then
		AchievementButton_ToggleTracking(frame.subaID)
		return
	end

	-- Handle Left Button click
	if button == "LeftButton" then
		local aID
		if frame.subaID then
			aID = frame.subaID
		else
			aID = frame.aID
		end

		if ( IsModifiedClick("CHATLINK") ) then
			local link = GetAchievementLink(aID)
			if ChatFrameEditBox:IsVisible() and link then
				ChatEdit_InsertLink(link)
			elseif link then
				SendChatMessage(link, DEFAULT_CHAT_FRAME.editBox:GetAttribute("chatType"))
			end
		elseif ( not AchievementFrame:IsShown() ) then
			AchievementFrame_ToggleAchievementFrame();
			AchievementFrame_SelectAchievement(aID);
		else
			if ( AchievementFrameAchievements.selection ~= aID ) then
				AchievementFrame_SelectAchievement(aID);
			else
				AchievementFrame_ToggleAchievementFrame();
			end
		end
		return
	end

	-- Handle
	if button == "RightButton"  then
		AchievementButton_ToggleTracking(frame.aID)
		return
	end

end

--[[
--------------------------------------------
-- Notes:
-- Shows Tooltip on Achievementtracker
--
-- Returns:
-- * nothing
--------------------------------------------]]
local AchievementTooltipID
--[[function aat:AchievementTrackerTooltip(frame)

	local aID, icon, subaID

	for i=1, #AchievementLines do
		if MouseIsOver(AchievementLines[i]) then
			aID = AchievementLines[i].aID
			if MouseIsOver(AchievementLines[i].iconframe) and AchievementLines[i].iconframe.aID then
				subaID = AchievementLines[i].iconframe.aID
				icon = true
			end
			if AchievementLines[i].subaID then
				subaID = AchievementLines[i].subaID
			end
			break
		end
	end

	if not aID then
		-- nothing to do
		aat.db.char.AchievementTracker_Tooltipset = nil;
		return
	end
	if
		frame.Tooltip.aID == aID
	and
		frame.Tooltip.Icon == icon
	and
		frame.Tooltip.SubAchievement == subaID
	then
		-- nothing to do
		return
	end

	aat.db.char.AchievementTracker_Tooltipset = true;
	frame.Tooltip.aID = aID
	frame.Tooltip.Icon = icon
	frame.Tooltip.SubAchievement = subaID

	AchievementTooltipID = aID

	if not aat.db.profile.showtooltip or not frame.LineFrame:IsShown() then
		return
	end

	if subaID then
		aID = subaID
	end
	-- show tooltip
	ResetCursor()
	ShowInspectCursor()
	frame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")

	local _, achievementName, _, completed, _, _, _, description = GetAchievementInfo(aID);
	frame.Tooltip:AddLine(achievementName)
	frame.Tooltip:AddLine(" ")
	frame.Tooltip:AddLine(description,1,1,1,true)

	local tooltiphint

	if aat.db.profile.showtooltiphint then

		if aat.db.profile.clickleft then
			tooltiphint = L['|cffeda55fClick|r to open achievement, |cffeda55fShift+Click|r to link achievement to chat.']
		end

		if aat.db.profile.clickright and not frame.Tooltip.Icon then
			tooltiphint = tooltiphint and tooltiphint .. L['|cffeda55fRight-Click|r to cancel tracking of this achievement'] or L['|cffeda55fRight-Click|r to cancel tracking of this achievement']
		elseif aat.db.profile.clickright and frame.Tooltip.Icon then
			tooltiphint = tooltiphint and tooltiphint .. L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"] or L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"]
		end

		if tooltiphint then
			frame.Tooltip:AddLine(tooltiphint,0,1,0,true)
		end
	end

	if aat.db.profile.debug then
		frame.Tooltip:AddLine("AchievementID: " .. aID,nil,nil,nil,true)
	end

	frame.Tooltip:Show()

end
--]]
--[[
function aat:AchievementTrackerTooltip(aID, subaID)
	
	if type(aID) ~= 'number' then return end
	
	local frame = aatAchievementWatchFrame
	
	if
		frame.Tooltip.aID == aID
--	and
--		frame.Tooltip.Icon == icon
	and
		frame.Tooltip.SubAchievement == subaID
	then
		-- nothing to do
		return
	end

	aat.db.char.AchievementTracker_Tooltipset = true;
	frame.Tooltip.aID = aID
	frame.Tooltip.Icon = icon
	frame.Tooltip.SubAchievement = subaID
	
	frame.aID = aID
	frame.subaID = subaID

	AchievementTooltipID = aID

	if not aat.db.profile.showtooltip or not frame.LineFrame:IsShown() then
		print("exit")
		return
	end

	if subaID then
		aID = subaID
	end
	-- show tooltip
	ResetCursor()
	ShowInspectCursor()
	frame.Tooltip:SetOwner(UIParent, "ANCHOR_CURSOR")

	local _, achievementName, _, completed, _, _, _, description = GetAchievementInfo(aID);
	frame.Tooltip:AddLine(achievementName)
	frame.Tooltip:AddLine(" ")
	frame.Tooltip:AddLine(description,1,1,1,true)

	local tooltiphint

	if aat.db.profile.showtooltiphint then

		if aat.db.profile.clickleft then
			tooltiphint = L['|cffeda55fClick|r to open achievement, |cffeda55fShift+Click|r to link achievement to chat.']
		end

		if aat.db.profile.clickright and not frame.Tooltip.Icon then
			tooltiphint = tooltiphint and tooltiphint .. L['|cffeda55fRight-Click|r to cancel tracking of this achievement'] or L['|cffeda55fRight-Click|r to cancel tracking of this achievement']
		elseif aat.db.profile.clickright and frame.Tooltip.Icon then
			tooltiphint = tooltiphint and tooltiphint .. L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"] or L["|cffeda55fRight-Click|r click to toggle watch for this Achievement"]
		end

		if tooltiphint then
			frame.Tooltip:AddLine(tooltiphint,0,1,0,true)
		end
	end

	if aat.db.profile.debug then
		frame.Tooltip:AddLine("AchievementID: " .. aID,nil,nil,nil,true)
	end

	frame.Tooltip:Show()

end--]]

function aat:round(input, places)
	if not places then
			places = 0
	end
	if type(input) == "number" and type(places) == "number" then
			local pow = 1
			for i = 1, ceil(places) do
					pow = pow * 10
			end
			return floor(input * pow + 0.5) / pow
	end
end

function aat:SaveFramePosition(frame)
	local posx, posy, point

	local x,y = frame:GetCenter()
	local hhalf = (x > UIParent:GetWidth()/2) and "RIGHT" or "LEFT"
	local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
	point = vhalf..hhalf

	posx = aat:round(hhalf == "LEFT" and (frame:GetLeft() * UIParent:GetScale()) or ((frame:GetLeft() - UIParent:GetWidth()) * UIParent:GetScale()) , 2)
	posy = aat:round(vhalf == "TOP" and ((frame:GetTop() - UIParent:GetHeight()) * UIParent:GetScale()) or (frame:GetTop() * UIParent:GetScale()) , 2)


	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", UIParent, point, posx/UIParent:GetScale(), posy/UIParent:GetScale())

	aat.db.char.FramePosition.posx = posx;
	aat.db.char.FramePosition.posy = posy;
	aat.db.char.FramePosition.point = point;
end

--- EOF ---
